<?php
/**
 * Функция add_vtbpay_gateway
 *
 * Добавляет платёжный метод "WC_Vtbpay" в список доступных платёжных методов WooCommerce.
 *
 * @param array $methods - Список доступных платёжных методов
 * @return array - Модифицированный список платёжных методов с добавлением WC_Vtbpay
 */
function add_vtbpay_gateway(array $methods): array
{
    $methods[] = 'WC_Vtbpay';
    return $methods;
}
add_filter('woocommerce_payment_gateways', 'add_vtbpay_gateway');


/**
 * Функция load_vtbpay_textdomain
 *
 * Загружает локализацию (текстовый перевод) для плагина WC_Vtbpay.
 */
function load_vtbpay_textdomain(): void
{
    load_plugin_textdomain('wc-vtbpay', false, dirname(plugin_basename(__FILE__)) . '/languages');
}
add_action('plugins_loaded', 'load_vtbpay_textdomain');


/**
 * Функция add_vtbpay_tab_custom_product_config
 *
 * Добавляет вкладку с настройками WC_Vtbpay к настройкам продукта в административной части WooCommerce.
 *
 * @param array $tabs - Список вкладок на странице редактирования продукта
 * @return array - Модифицированный список вкладок с добавлением WC_Vtbpay
 */
function add_vtbpay_tab_custom_product_config(array $tabs): array
{
    // Проверяет наличие настроек WC_Vtbpay и условие активации для добавления вкладки
    $vtbpay_settings = get_option('woocommerce_wc_vtbpay_settings');

    if (
        isset($vtbpay_settings['cashiers_check_enable_1ofd']) &&
        $vtbpay_settings['cashiers_check_enable_1ofd'] === 'yes'
    ) {
        // Добавляет вкладку с настройками WC_Vtbpay
        $tabs['custom_tab'] = [
            'label'    => __('Payment system VTB', 'wc-vtbpay'),
            'target'   => 'vtbpay_custom_product_data',
            'class'    => ['show_if_simple', 'show_if_variable'],
        ];
    }

    return $tabs;
}
add_filter('woocommerce_product_data_tabs', 'add_vtbpay_tab_custom_product_config');


/**
 * Функция create_custom_container_vtbpay_settings
 *
 * Создает контейнер с настройками WC_Vtbpay на странице редактирования продукта в административной части WooCommerce.
 */
function create_custom_container_vtbpay_settings(): void
{
    // Проверяет наличие настроек WC_Vtbpay и условие активации для отображения контейнера
    $vtbpay_settings = get_option('woocommerce_wc_vtbpay_settings');

    if (
        isset($vtbpay_settings['cashiers_check_enable_1ofd']) &&
        $vtbpay_settings['cashiers_check_enable_1ofd'] === 'yes'
    ) {
        echo '<div id="vtbpay_custom_product_data" class="panel woocommerce_options_panel">';

        // Определяет поля настроек и добавляет их на страницу редактирования продукта
        $fields = [
            'vtbpay_payment_type_1ofd' => __('Type of calculation', 'wc-vtbpay'),
            'vtbpay_payment_subject_1ofd' => __('Subject of calculation', 'wc-vtbpay'),
            'vtbpay_vat_code_1ofd' => __('Tax rate', 'wc-vtbpay'),
        ];

        foreach ($fields as $field_id => $field_label) {
            woocommerce_wp_select([
                'id' => $field_id,
                'label' => $field_label,
                'options' => get_vtbpay_field_options($field_id),
            ]);
        }

        echo '</div>';
    }
}
add_action('woocommerce_product_data_panels', 'create_custom_container_vtbpay_settings');


/**
 * Функция get_vtbpay_field_options
 *
 * Возвращает список доступных опций для полей настроек WC_Vtbpay на основе идентификатора поля.
 *
 * @param string $field_id - Идентификатор поля настроек
 * @return array - Список опций для поля
 */
function get_vtbpay_field_options($field_id): array
{
    $options = [
        '-' => __('Select an option', 'wc-vtbpay'),
    ];

    // Определяет опции для конкретного поля
    switch ($field_id) {
        case 'vtbpay_payment_type_1ofd':
            $options = array_merge($options, [
                'FULL_PREPAYMENT' => __('Pre-payment 100%', 'wc-vtbpay'),
                'PARTIAL_PREPAYMENT' => __('Pre-payment', 'wc-vtbpay'),
                'ADVANCE' => __('Advance', 'wc-vtbpay'),
                'FULL_PAYMENT' => __('Full payment', 'wc-vtbpay'),
                'PARTIAL_PAYMENT' => __('Partial settlement and credit', 'wc-vtbpay'),
                'CREDIT' => __('Transfer on credit', 'wc-vtbpay'),
                'CREDIT_PAYMENT' => __('Loan repayment', 'wc-vtbpay')
            ]);
            break;

        case 'vtbpay_payment_subject_1ofd':
            $options = array_merge($options, [
                'GOODS' => __('Goods', 'wc-vtbpay'),
                'EXCISE_GOODS' => __('Excisable goods', 'wc-vtbpay'),
                'JOB' => __('Job', 'wc-vtbpay'),
                'SERVICE' => __('Service', 'wc-vtbpay'),
                'GAMBLING_BET' => __('Betting gambling', 'wc-vtbpay'),
                'GAMBLING_WIN' => __('Winning gambling', 'wc-vtbpay'),
                'LOTTERY_TICKET' => __('Lottery Bet', 'wc-vtbpay'),
                'LOTTERY_WIN' => __('Lottery winnings', 'wc-vtbpay'),
                'RID_ACCESS' => __('RID Provision', 'wc-vtbpay'),
                'ADVANCE' => __('Payment', 'wc-vtbpay'),
                'AGENT_REWARD' => __('Agency fee', 'wc-vtbpay'),
                'COMPOSITE_11' => __('Disbursement', 'wc-vtbpay'),
                'COMPOSITE_NONE_11' => __('Other subject matter', 'wc-vtbpay'),
                'PROPERTY_LAW' => __('Property law', 'wc-vtbpay'),
                'NONE_OPERATIONAL_INCOME' => __('Non-operating income', 'wc-vtbpay'),
                'TAX_REDUCTION' => __('Insurance premiums', 'wc-vtbpay'),
                'TRADE_FEE' => __('Trade fee', 'wc-vtbpay'),
                'RESORT_FEE' => __('Resort fee', 'wc-vtbpay'),
                'CAUTION_MONEY' => __('Bail', 'wc-vtbpay'),
                'EXPENSE_W_INCOME_REDUCTION' => __('Expenditure', 'wc-vtbpay'),
                'PENSION_INSUARACE' => __('Contributions to IE\'s OPS', 'wc-vtbpay'),
                'PENSION_INSUARACE_W_PAYMENTS' => __('OPS contributions', 'wc-vtbpay'),
                'MEDICAL_INSUARACE' => __('Contributions to IE MHI', 'wc-vtbpay'),
                'MEDICAL_INSUARACE_W_PAYMENTS' => __('Contributions to MHI', 'wc-vtbpay'),
                'SOCIAL_INSUARANCE' => __('Contributions to the OSS', 'wc-vtbpay'),
                'CASINO' => __('Casino payment', 'wc-vtbpay')
            ]);
            break;

        case 'vtbpay_vat_code_1ofd':
            $options = array_merge($options, [
                'NONE' => __('No tax rate', 'wc-vtbpay'),
                'VAT_GENERAL' => __('VAT 20%', 'wc-vtbpay'),
                'VAT_PREFERENTIAL' => __('VAT 10%', 'wc-vtbpay'),
                'VAT_GENERAL_CALC' => __('VAT 20/120', 'wc-vtbpay'),
                'VAT_PREFERENTIAL_CALC' => __('VAT 10/110', 'wc-vtbpay'),
                'VAT_ZERO' => __('VAT 0%', 'wc-vtbpay')
            ]);
            break;
    }

    return $options;
}


/**
 * Функция save_value_custom_product_fields_from_vtbpay_tab
 *
 * Сохраняет значения пользовательских полей настроек WC_Vtbpay на странице редактирования продукта.
 *
 * @param int $post_id - Идентификатор продукта
 */
function save_value_custom_product_fields_from_vtbpay_tab($post_id): void
{
    // Проверяет наличие настроек WC_Vtbpay и условие активации для сохранения полей
    $vtbpay_settings = get_option('woocommerce_wc_vtbpay_settings');

    if (
        isset($vtbpay_settings['cashiers_check_enable_1ofd']) &&
        $vtbpay_settings['cashiers_check_enable_1ofd'] === 'yes'
    ) {
        // Определяет поля, получает их значения из запроса и сохраняет в метаданных продукта
        $fields = [
            'vtbpay_payment_type_1ofd',
            'vtbpay_payment_subject_1ofd',
            'vtbpay_vat_code_1ofd',
        ];

        foreach ($fields as $field) {
            $value = sanitize_text_field($_POST[$field] ?? '');
            update_post_meta($post_id, $field, $value);
        }
    }
}
add_action('woocommerce_process_product_meta', 'save_value_custom_product_fields_from_vtbpay_tab');


/**
 * Функция adding_vtbpay_button_to_orders_page
 *
 * Добавляет скрипты и стили к странице заказов и настройкам WC_Vtbpay в административной части.
 *
 * @param string $hook - Идентификатор страницы в административной части WordPress
 */
function adding_vtbpay_button_to_orders_page($hook): void
{
    // Проверяет наличие настроек WC_Vtbpay и условия для добавления скриптов и стилей
    $vtbpay_settings = get_option('woocommerce_wc_vtbpay_settings');
    global $post;

    if (
        isset($vtbpay_settings['cashiers_check_enable_1ofd']) &&
        $vtbpay_settings['cashiers_check_enable_1ofd'] === 'yes' &&
        ($hook == 'post-new.php' || $hook == 'post.php') &&
        !empty($post) && $post->post_type === 'shop_order'
    ) {
        $order_id = $post->ID;
        $order = wc_get_order($order_id);
        $order_status = $order->get_status();
        $payment_method = $order->get_payment_method();

        if (
            $order_status !== 'failed' &&
            $order_status !== 'refunded' &&
            $order_status !== 'cancelled' &&
            $payment_method === 'wc_vtbpay'
        ) {
            // Подключает скрипт для работы с заказами WC_Vtbpay
            wp_enqueue_script('vtbpay-order-page-script', plugins_url('/assets/js/vtbpay-order-page-script.js', __FILE__), ['jquery'], '1.0', true);
            wp_localize_script(
                'vtbpay-order-page-script',
                'vtbpay_ajax_var', [
                'nonce' => wp_create_nonce('vtbpay-ajax-nonce'),
                'api_url' => home_url('/wc-api/wc_vtbpay_check_and_edit_order_status')
            ]);
        }
    }

    if (
        $hook == 'woocommerce_page_wc-settings' &&
        isset($_GET['tab']) && $_GET['tab'] === 'checkout' &&
        isset($_GET['section']) && $_GET['section'] === 'wc_vtbpay'
    ) {
        // Подключает скрипт для страницы настроек WC_Vtbpay
        wp_enqueue_script('vtbpay-settings-page-script', plugins_url('/assets/js/vtbpay-settings-page-script.js', __FILE__), ['jquery'], '1.0', true);
        wp_enqueue_style('vtbpay-settings-page-style', plugins_url('/assets/css/vtbpay-settings-page-style.css', __FILE__));
    }
}
add_action('admin_enqueue_scripts', 'adding_vtbpay_button_to_orders_page');
