<?php

namespace Vtbpay\Classes\Api;

(defined('ABSPATH') || PHP_SAPI == 'cli') or die('Restricted access');

use \Vtbpay\Classes\Common\Request,
    \Vtbpay\Classes\Exception\VtbPayException;

/**
 * Класс FirstOFDApi предоставляет методы для взаимодействия с API Первого ОФД.
 */
class FirstOFDApi extends Request
{
    private string $mode;
    
    private string $accessToken;
    
    private array $clientData = [];
    
    private static array $modeUrls = [
        'PROD' => [
            'TOKEN' => 'https://external-api-farm.1-ofd.ru/getToken',
            'RECEIPT' => 'https://external-api-farm.1-ofd.ru/api/external/queue/v1/transaction/receipt',
            'RECEIPT_CORRECTION' => 'https://external-api-farm.1-ofd.ru/api/external/queue/v1/transaction/receiptCorrection'
        ],
        'TEST' => [
            'TOKEN' => 'https://external-api-farm.1-ofd-test.ru/getToken',
            'RECEIPT' => 'https://external-api-farm.1-ofd-test.ru/api/external/queue/v1/transaction/receipt',
            'RECEIPT_CORRECTION' => 'https://external-api-farm.1-ofd-test.ru/api/external/queue/v1/transaction/receiptCorrection'
        ]
    ];
  
    
    /**
     * Конструктор класса FirstOFDApi.
     *
     * @param string $login       Логин интеграции Первого ОФД.
     * @param string $pass        Пароль интеграции Первого ОФД (Мастер-ключ API).
     * @param string $INN         ИНН организации мерчанта.
     * @param string $taxSystem   Система налогообложения.
     * @param string $groupIdKKT  Идентификатор группы ККТ.
     * @param bool $test          Режим тестирования (по умолчанию - false).
     *
     * @throws VtbPayException Если не заполнены обязательные поля.
     */
    public function __construct(
        string $login,
        string $pass,
        string $INN,
        string $taxSystem,
        string $groupIdKKT,
        bool $test = false
    ) {
        if (
            empty($login) || 
            empty($pass) || 
            empty($INN) || 
            empty($groupIdKKT)
        ) throw new VtbPayException('Обязательные поля не заполнены.');
        
        $this->mode = $test ? 'TEST': 'PROD';
        
        $this->accessToken = $this->getAccessToken(
            trim($login), 
            trim($pass)
        );
        
        $this->clientData = [
            'INN' => trim($INN),
            'groupIdKKT' => trim($groupIdKKT),
            'taxSystem' => $taxSystem
        ];
    }
    
    
    /**
     * Получает URL для заданного типа запроса.
     *
     * @param string $type Тип запроса ('TOKEN', 'RECEIPT' или 'RECEIPT_CORRECTION').
     *
     * @return string URL для запроса.
     */
    private function getUrl(string $type): string
    {
        return self::$modeUrls[$this->mode][$type] ?? '';
    }
    
    
    /**
     * Делаем запрос в Первый ОФД для полученя токена и возвращаем его.
     *
     * @param string $login   Логин интеграции Первый ОФД
     * @param string $pass    Пароль интеграции Первый ОФД (Мастер-ключ API)
     *
     * @return string URL для запроса.
     */
    private function getAccessToken(string $login, string $pass): string 
    {
        // getToken request
        $response = $this->sendRequest(
            $this->getUrl('TOKEN'), [
                'login' => $login,
                'pass' => $pass
            ], [
                'Content-type: application/json; charset=utf-8'
            ]
        );
        
        $this->validateResponseAccessToken($response);
        
        return $response['token'];
    }
    
    
    /**
     * Проверяем результат запроса в Первый ОФД на получение токена.
     *
     * @param array $response Массив резульата запроса
     *
     * @return void|VtbPayException Выбрасываем ошибку если токена нет.
     */
    private function validateResponseAccessToken(array $response): void
    {
        if (
            !isset($response['token']) || 
            empty($response['token'])
        ) {
            throw new VtbPayException('Ошибка при получении токена доступа.', [
                'response' => $response
            ]);
        }
    }
    
    
    /**
     * Запрос на добавление чека в очередь печати.
     *
     * @param string $orderId   Идентификатор заказа
     * @param array $items      Позиции чека
     * @param float $total      Сумма заказа
     * @param string $name      Полное имя клиента
     * @param string $mail      Электронная почта клиента
     *
     * @return array Результат запроса
     */
    public function setReceipt(
        string $orderId, 
        array $items, 
        float $total, 
        string $name, 
        string $mail
    ): array {
        $response = $this->sendRequest(
            $this->getUrl('RECEIPT'), [
                'requestMetadata' => [
                    'userGroup' => $this->clientData['groupIdKKT'],
                    'timestamp' => date('d.m.Y H:i:s', time()),
                    'externalId' => $orderId
                ],
                'document' => [
                    'operationType' => 'INCOME',
                    'items' => $items,
                    'taxationType' => $this->clientData['taxSystem'],
                    'totalSum' => [
                        'ecashTotalSum' => (float) $total
                    ],
                    'sendCheckTo' => $mail,
                    'buyer' => [
                        'name' => $name,
                        'email' => $mail,
                    ]
                ],
                'cashboxParameters' => [
                    'userInn' => $this->clientData['INN']
                ]
            ], [
                'Content-type: application/json; charset=utf-8',
                'Authorization: ' . $this->accessToken
            ]
        );
        
        $this->validateResponseSetReceipt($response);
        
        return $response;
    }
    
    
    /**
     * Проверяем результат запроса на добавление чека в очередь печати.
     *
     * @param array $response Массив резульата запроса
     *
     * @return void|VtbPayException Выбрасываем ошибку если статус не COMPLETED.
     */
    private function validateResponseSetReceipt(array $response): void
    {
        if (
            !isset($response['status']) || 
            $response['status'] !== 'COMPLETED'
        ) {
            throw new VtbPayException('Ошибка при отправлении чека.', [
                'response' => $response
            ]);
        }
    }
    
    
    /**
     * Запрос на добавление коррекции чека в очередь печати.
     *
     * @param string $orderId   Идентификатор заказа
     * @param array $items      Позиции чека
     * @param float $total      Сумма заказа
     * @param string $name      Полное имя клиента
     * @param string $mail      Электронная почта клиента
     *
     * @return array Результат запроса
     */
    public function setReceiptCorrection(
        string $orderId, 
        array $items, 
        float $total, 
        string $name, 
        string $mail
    ): array {
        $response = $this->sendRequest(
            $this->getUrl('RECEIPT_CORRECTION'), [
                'requestMetadata' => [
                    'userGroup' => $this->clientData['groupIdKKT'],
                    'timestamp' => date('d.m.Y H:i:s', time()),
                    'externalId' => $orderId . '-REFUND-' . time()
                ],
                'document' => [
                    'correctionCause' => [
                        'correctionType' => 'SELF',
                        'correctionDocDate' => date('Y-m-d\TH:i:sP', time())
                    ],
                    'operationType' => 'INCOME_RETURN',
                    'items' => $items,
                    'taxationType' => $this->clientData['taxSystem'],
                    'totalSum' => [
                        'ecashTotalSum' => (float) $total
                    ],
                    'sendCheckTo' => $mail,
                    'buyer' => [
                        'name' => $name,
                        'email' => $mail,
                    ]
                ],
                'cashboxParameters' => [
                    'userInn' => $this->clientData['INN']
                ]
            ], [
                'Content-type: application/json; charset=utf-8',
                'Authorization: ' . $this->accessToken
            ]
        );
        
        $this->validateResponseSetReceiptCorrection($response);
        
        return $response;
    }
    
    
    /**
     * Проверяем результат запроса на добавление коррекции чека в очередь печати.
     *
     * @param array $response Массив резульата запроса
     *
     * @return void|VtbPayException Выбрасываем ошибку если статус не COMPLETED.
     */
    private function validateResponseSetReceiptCorrection(array $response): void
    {
        if (
            !isset($response['status']) || 
            $response['status'] !== 'COMPLETED'
        ) {
            throw new VtbPayException('Ошибка при отправлении коррекции чека.', [
                'response' => $response
            ]);
        }
    }
}