<?php
/**
 * Plugin Name: Payment system VTB
 * Description: Allows you to use Payment system VTB with the WooCommerce plugin.
 * Version: 1.0.4
 * Author: VTB
 * Author URI: https://www.vtb.ru/
 * Text Domain: wc-vtbpay
 * Domain Path: /languages/
 * Requires PHP: 7.4
 *
 * @package Vtbpay
 * @version 1.0.4
 */

if (!defined('ABSPATH')) exit; // Exit if accessed directly

require __DIR__ . '/vendor/autoload.php';
require __DIR__ . '/hooks.php';

use \Vtbpay\Classes\Api\VtbApi,
    \Vtbpay\Classes\Api\FirstOFDApi,
    \Vtbpay\Classes\Common\VtbPayLogger,
    \Vtbpay\Classes\Exception\VtbPayException;


add_action('plugins_loaded', 'init_wc_vtb_payment_gateway');

/**
 * Initialize Payment system VTB.
 */
function init_wc_vtb_payment_gateway(): void
{
    // if the WC payment gateway class is not available, do nothing
    if (!class_exists('WC_Payment_Gateway') || class_exists('WC_Vtbpay')) return;

    /**
     * Payment system VTB class.
     */
    class WC_Vtbpay extends WC_Payment_Gateway
    {
        // Payment system VTB
        private string $client_id_vtbpay;
        private string $client_secret_vtbpay;
        private string $merchant_authorization_vtbpay;
        private bool $test_mode_vtbpay;
        private bool $logging_vtbpay;
        private string $transaction_end_vtbpay;

        // First OFD
        private bool $cashiers_check_enable_1ofd;
        private bool $cancel_order_in_receipt_error_1ofd;
        private bool $test_mode_1ofd;
        private string $login_1ofd;
        private string $pass_1ofd;
        private string $INN_1ofd;
        private string $email_1ofd;
        private string $tax_system_1ofd;
        private string $group_id_KKT_1ofd;
        private string $payment_type_1ofd;
        private string $payment_type_delivery_1ofd;
        private string $payment_subject_1ofd;
        private string $vat_code_1ofd;

        // Common
        private object $logger;
        private object $order;
        private ?object $vtb_api_obj = null;
        private ?object $firstOFD_api_obj = null;


        /**
         * Конструктор для класса плагина "Платежная система ВТБ".
         */
        function __construct()
        {
            $plugin_dir = plugin_dir_url(__FILE__);

            $this->id = 'wc_vtbpay';
            $this->icon = apply_filters('woocommerce_vtbpay_icon', $plugin_dir . 'vtbpay.png');
            $this->method_title = __('Payment system VTB', 'wc-vtbpay');
            $this->method_description = __('Plugin "VTB Payment System" for WooCommerce, which allows you to integrate online payments.', 'wc-vtbpay');
            $this->has_fields = false;

            // Load the settings
            $this->init_form_fields();
            $this->init_settings();

            // Define user set variables.
        		$this->title        = $this->get_option('title');
        		$this->description  = $this->get_option('description');
        		$this->instructions = $this->get_option('instructions');
            $this->enabled      = $this->get_option('enabled');

            // Initialize settings
            $this->init_payment_settings();

            $this->set_vtbpay_logger();

            //process settings with parent method
            add_action('woocommerce_update_options_payment_gateways_' . $this->id, [$this, 'process_admin_options']);
            add_action('woocommerce_api_' . $this->id . '_webhook', [$this, 'webhook']);
            // handler for checking and editing order status.
            add_action('woocommerce_api_' . $this->id . '_check_and_edit_order_status', [$this, 'check_and_edit_order_status_handler']);
        }


        /**
         * Инициализация настроек
         *
         * @return void
         */
        private function init_payment_settings(): void
        {
            // Settings
            $settings = [
                'client_id_vtbpay',
                'client_secret_vtbpay',
                'merchant_authorization_vtbpay',
                'transaction_end_vtbpay',
            ];

            foreach ($settings as $setting) {
                $this->$setting = $this->get_option($setting);
            }

            // Booleans
            $booleanSettings = [
                'test_mode_vtbpay',
                'logging_vtbpay',
                'cashiers_check_enable_1ofd',
                'cancel_order_in_receipt_error_1ofd',
                'test_mode_1ofd'
            ];

            foreach ($booleanSettings as $setting) {
                $this->$setting = $this->get_option($setting) === 'yes';
            }

            // First OFD settings
            $firstOFDSettings = [
                'login_1ofd',
                'pass_1ofd',
                'INN_1ofd',
                'email_1ofd',
                'tax_system_1ofd',
                'group_id_KKT_1ofd',
                'payment_type_1ofd',
                'payment_type_delivery_1ofd',
                'payment_subject_1ofd',
                'vat_code_1ofd',
            ];

            foreach ($firstOFDSettings as $setting) {
                $this->$setting = $this->get_option($setting);
            }
        }


        /**
         * Инициализируйте поля формы настроейк платежной системы ВТБ.
         *
         * @return void
         */
        public function init_form_fields(): void
        {
            $this->form_fields = include 'form_fields.php';
        }


        /**
         * Инициализируем свойство order объектом класса WC_Order,
         * полученным с помощью идентификатора и возвращаем объект текущего класса.
         *
         * @param int $order_id Идентификатор заказа.
         *
         * @return self|Exception Объект заказа, если он существует, или Exception в противном случае.
         */
        private function set_order(int $order_id): self
        {
            if ($order = wc_get_order($order_id)) {
                $this->order = $order;
                return $this;
            }
            else {
                throw new \Exception(__('Order not found.', 'wc-vtbpay'));
            }
        }


        /**
         * Создаем новый экземпляр класса VtbApi с заданными конфигурациями платёжной системы.
         *
         * @return VtbApi The new VtbApi instance.
         */
        protected function get_vtb_api(): VtbApi
        {
            if ($this->vtb_api_obj === null) {
                $this->vtb_api_obj = new VtbApi(
                    $this->client_id_vtbpay,
                    $this->client_secret_vtbpay,
                    $this->test_mode_vtbpay,
                    $this->merchant_authorization_vtbpay
                );
            }

            return $this->vtb_api_obj;
        }


        /**
         * Инициализируем свойство firstOFD_api_obj объектом класса FirstOFDApi
         * и возвращаем данный объект
         *
         * @return FirstOFDApi Экземпляр класса FirstOFDApi.
         */
        protected function get_firstOFD_api(): FirstOFDApi
        {
            if ($this->firstOFD_api_obj === null) {
                $this->firstOFD_api_obj = new FirstOFDApi(
                    $this->login_1ofd,
                    $this->pass_1ofd,
                    $this->INN_1ofd,
                    $this->tax_system_1ofd,
                    $this->group_id_KKT_1ofd,
                    $this->test_mode_1ofd
                );
            }

            return $this->firstOFD_api_obj;
        }


        /**
         * Обработка оплаты заказа в WooCommerce с помощью платежной системы ВТБ.
         * должно быть совместимо с WC_Payment_Gateway::process_payment($order_id)
         *
         * @param int $order_id Идентификатор заказа.
         *
         * @return array|null Массив, содержащий результат и URL перенаправления, или null, если произошла ошибка.
         */
        public function process_payment($order_id): ?array
        {
            try {
                $this->set_order($order_id);

                if ($this->order->get_payment_method() != $this->id) {
                    throw new \Exception(__('Payment method not "VTB Payment System"', 'wc-vtbpay'));
                }

                $pay_url = $this->get_pay_url();

                //Set order status pending payment
                $this->order->update_status('pending', __('Payment link generated:', 'wc-vtbpay') . $pay_url);

                //once the order is updated clear the cart and reduce the stock
                WC()->cart->empty_cart();

                return [
                    'result' => 'success',
                    'redirect' => $pay_url
                ];

            } catch (\Exception | VtbPayException $e) {
                // Handle exception and log error
                $context = [];
                if (method_exists($e, 'getContext')) $context = $e->getContext();

                $this->logger->error(sprintf(
    								__FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
    								$e->getMessage(),
    								$order_id ?: ''
    						), $context);

                wc_add_notice($e->getMessage(), 'error');
                return null;
            }
        }


        /**
         * Получает URL-адрес платежа для перенаправления.
         *
         * @return string URL-адрес платежа.
         */
        private function get_pay_url(): string
        {
            $order_id = $this->order->get_id();
            $email = $this->order->get_billing_email();
            $total = $this->order->get_total();
            $return_url = home_url(sprintf('/wc-api/%s_webhook', $this->id)); // Get return URL

    				// Logging input
    				$this->logger->debug(
                __FUNCTION__ . ' > getOrderLink - INPUT: ', [
    						'arguments' => [
    								'order_id' => $order_id,
    								'email' => $email,
    								'time' => time(),
    								'total' => $total,
    								'return_url' => $return_url
    						]
    				]);

            $response = $this->get_vtb_api()->getOrderLink(
                $order_id,
                $email,
                time(),
                $total,
                $return_url
            );

    				// Logging output
    				$this->logger->debug(
                __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
    						'response' => $response
    				]);

            return $response['object']['payUrl'] ?? '';
        }


        /**
         * Отображение описания платежной системы ВТБ при оформлении заказа.
         *
         * @return void
         */
        public function payment_fields(): void
        {
            if (!empty($this->description)) {
                echo wpautop(wptexturize($this->description));
            }
        }


        /**
         * Обработчик webhook, вызываемый при переходе на страницу return_url, после попытки оплаты.
         *
         * @return void
         */
        public function webhook(): void
        {
            $order_id = $_REQUEST['orderId'] ?? null;

            try {
                if (empty($order_id)) {
                    throw new \Exception(__('Order ID is empty.', 'wc-vtbpay'));
                }

                $this->set_order($order_id);

                // Check order not already completed
                if ($this->order->get_status() == 'completed') {
                    throw new \Exception(__('Order already paid.', 'wc-vtbpay'));
                }

                if ($this->get_payment_status() === 'PAID') {
                    // Payment completed
                    $this->actions_for_paid_order();
                    $redirect_url = $this->get_return_url($this->order);
                    if($this->cashiers_check_enable_1ofd) $this->sendReceiptInFirstOFD();
                }
                else {
                    // Payment not completed
                    $this->actions_for_unpaid_order();
                    $redirect_url = str_replace('&amp;', '&', $this->order->get_cancel_order_url());
                }

                wp_safe_redirect(esc_url($redirect_url));

            } catch (\Exception | VtbPayException $e) {
                // Handle exception and log error
                $context = [];
                if (method_exists($e, 'getContext')) $context = $e->getContext();

                $this->logger->error(sprintf(
                    __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                    $e->getMessage(),
                    $order_id ?: ''
                ), $context);

                wp_die($e->getMessage());
            }
        }


        /**
         * Отправляем чек на печать в Первый ОФД.
         * Если во время отправки чека произошла ошибка, то смотрим на свойство cancel_order_in_receipt_error_1ofd.
         * Если cancel_order_in_receipt_error_1ofd == true,
         * то делаем возврат в ВТБ и отображаем страницу ошибки
         * Если cancel_order_in_receipt_error_1ofd == false,
         * то отправляем письмо о неудаче на почту из свойства email_1ofd и ошибку не отображаем
         *
         * @return void|Exception
         */
        private function sendReceiptInFirstOFD(): void
        {
            $order_id = $this->order->get_id();

            try {
                $this->sendReceipt();

            } catch (\Exception | VtbPayException $e) {

                if ($this->cancel_order_in_receipt_error_1ofd) {

                    // Делаем возврат платежа
                    $this->makeChargebackInVtbpay();

                    throw $e;
                }
                else {
                  // Отправляем уведомление об ошибке
                  $this->sendErrorEmail();
                }
            }
        }


        /**
         * Отправляем запрос на возврат в ВТБ, возвращвем результат запроса и логируем входящие и выходящие данные.
         * Если запрос прошёл успешно, то и в CMS отображаем информацию о возврате.
         *
         * @return array
         */
        private function makeChargebackInVtbpay(): array
        {
            $order_id = $this->order->get_id();

            // Logging input
            $this->logger->debug(
                __FUNCTION__ . ' > setRefunds - INPUT: ', [
                'arguments' => [
                    'order_id' => $order_id
                ]
            ]);

            $response = $this->get_vtb_api()->setRefunds($order_id);

            // Logging output
            $this->logger->debug(
                __FUNCTION__ . ' > setRefunds - OUTPUT: ', [
                'response' => $response
            ]);

            // Payment refunding
            $this->actions_for_refunded_order();

            return $response;
        }


        /**
         * Отправляем письмо с ошибкой отправки чека на печать в Первый ОФД
         * на почту из свойства email_1ofd
         *
         * @return void
         */
        private function sendErrorEmail(): void
        {
            $site_name = get_bloginfo('name'); // Получаем название сайта

            // Задаем параметры для отправки письма
            $headers = [
                'From: ' . $site_name . ' <' . get_option('admin_email') . '>',
                'Content-Type: text/html; charset=UTF-8',
            ];

            $subject = __('Check issuance error.', 'wc-vtbpay');
            $message = sprintf(
                __('There was an error when creating a check for order № %s. Please generate a check yourself in your personal "First OFD" cabinet.', 'wc-vtbpay'),
                $this->order->get_id()
            );

            // Отправляем письмо
            wp_mail(
                $this->email_1ofd,
                $subject,
                $message,
                $headers
            );
        }


        /**
         * Проверяем статус платежа, и если он PAID, то устанавливаем заказ в CMS как оплаченный.
         * Если статус платежа REFUNDED, то отправляем в Первый ОФД скорректированный чек,
         * в котором указано, что он отменён и в CMS устанавливаем заказ в как отменённый.
         *
         * @return void
         */
        public function check_and_edit_order_status_handler(): void
        {
            // Получите данные из AJAX-запроса через $_POST
            $order_id = $_POST['orderId'] ?? '';

            try {
                if (!wp_verify_nonce($_POST['nonce'], 'vtbpay-ajax-nonce')) {
                    throw new \Exception(__('Failed ajax validation.', 'wc-vtbpay'));
                }

                if (empty($order_id)) {
                    throw new \Exception(__('Order ID is empty.', 'wc-vtbpay'));
                }

                $this->set_order($order_id);

                $payment_status = $this->get_payment_status();

                if (
                    $payment_status === 'PAID' &&
                    $this->order->get_status() !== 'completed'
                ) {
                    // Payment completed
                    $this->actions_for_paid_order();

                    $message = __('Payment completed successfully.', 'wc-vtbpay');
                }
                elseif ($payment_status === 'REFUNDED') {

                    $this->sendReceipt(true);

                    // Payment refunding
                    $this->actions_for_refunded_order();

                    $message = __('The payment was refunded. The payment for the order was refunded through the personal account.', 'wc-vtbpay');
                }
                else {
                    throw new \Exception('There\'s nothing to update.');
                }

                // Отправьте JSON-ответ и завершите выполнение скрипта
                wp_send_json([
                    'success' => true,
                    'message' => $message
                ]);

            } catch (\Exception | VtbPayException $e) {
                // Handle exception and log error
                $context = [];
                if (method_exists($e, 'getContext')) $context = $e->getContext();

                $this->logger->error(sprintf(
                    __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                    $e->getMessage(),
                    $order_id ?: ''
                ), $context);

                wp_send_json([
                  'success' => false,
                  'message' => $e->getMessage()
                ]);
            }
        }


        /**
         * Отправляем в Первый ОФД чек на печать или коррекцию чека и возвращаем результат.
         *
         * @param bool $is_correction Если false, то отправка чека на печать, если true, то отправка коррекции чека
         *
         * @return array
         */
        private function sendReceipt(bool $is_correction = false): array
        {
            $func_name = 'setReceipt';
            if ($is_correction) $func_name = 'setReceiptCorrection';

            $order_id = $this->order->get_id();

            $full_name = $this->order->get_billing_first_name() . ' ' . $this->order->get_billing_last_name();

            $email = $this->order->get_billing_email() ?: $this->email_1ofd;

            $total = $this->order->get_total();

            $items = $this->get_formatted_order_items();

            // Logging input
            $this->logger->debug(
                __FUNCTION__ . " > {$func_name} - INPUT: ", [
                'arguments' => [
                    'order_id' => $order_id,
                    'items' => $items,
                    'total' => $total,
                    'full_name' => $full_name,
                    'email' => $email
                ]
            ]);

            $response = $this->get_firstOFD_api()->$func_name(
                $order_id,
                $items,
                $total,
                $full_name,
                $email
            );

            // Logging output
            $this->logger->debug(
                __FUNCTION__ . " > {$func_name} - OUTPUT: ", [
                'response' => $response
            ]);

            return $response;
        }


        /**
         * Форматируем позиции чека (товары, доставка) в нужный вид
         * перед отправкой в Первый ОФД и возвращаем результат.
         *
         * @return array
         */
        private function get_formatted_order_items(): array
        {
            $final_items = [];

            foreach ($this->order->get_items() as $item_id => $item) {

                $product_id = $item->get_product_id();

                $product = $item->get_product();

                $price = wc_get_price_including_tax($product);

                $amount = wc_get_price_including_tax(
                  $product, [
                      'qty' => $item->get_quantity()
                ]);

                $vat_code = get_post_meta($product_id, 'vtbpay_vat_code_1ofd', true);
                if (empty($vat_code) || $vat_code === '-') $vat_code = $this->vat_code_1ofd;

                $payment_type = get_post_meta($product_id, 'vtbpay_payment_type_1ofd', true);
                if (empty($payment_type) || $payment_type === '-') $payment_type = $this->payment_type_1ofd;

                $payment_subject = get_post_meta($product_id, 'vtbpay_payment_subject_1ofd', true);
                if (empty($payment_subject) || $payment_subject === '-') $payment_subject = $this->payment_subject_1ofd;

                $final_items[] = [
                    'label' => $item->get_name(),
                    'price' => $price,
                    'quantity' => $item->get_quantity(),
                    'vatCode' => $vat_code,
                    'amount' => $amount,
                    'paymentType' => $payment_type,
                    'paymentSubject' => $payment_subject
                ];
            }

            // Получаем стоимость доставки
            $shipping_cost = $this->order->get_shipping_total();

            // Если стоимость доставки больше 0, добавляем ее в массив
            if ($shipping_cost > 0) {
                $final_items[] = [
                    'label' => __('Delivery', 'wc-vtbpay'),
                    'price' => $shipping_cost,
                    'quantity' => 1,
                    'vatCode' => $vat_code,
                    'amount' => $shipping_cost,
                    'paymentType' => $this->payment_type_delivery_1ofd,
                    'paymentSubject' => 'SERVICE'
                ];
            }

            return $final_items;
        }


        /**
         * Обработка действий для оплаченного заказа.
         *
         * @return void
         */
        private function actions_for_paid_order(): void
        {
            $this->order->update_status(
                str_replace('wc-', '', $this->transaction_end_vtbpay),
                __('Payment completed successfully.', 'wc-vtbpay')
            );
            $this->order->payment_complete();
            wc_reduce_stock_levels($this->order->get_id());
        }


        /**
         * Обработка действий в случае неоплаченного заказа.
         *
         * @return void
         */
        private function actions_for_unpaid_order(): void
        {
            // Logging an unpaid order
            $this->logger->debug(sprintf(
                __FUNCTION__ . '. ' . __('Payment not paid.', 'wc-vtbpay') . '; Order ID: %s',
                $this->order->get_id()
            ));

            $this->order->update_status('failed', __('Payment not paid.', 'wc-vtbpay'));
            wc_add_notice(__('Payment not paid. Your order has been canceled.', 'wc-vtbpay'), 'error');
        }


        /**
         * Обработка действий в случае возвращённого заказа.
         *
         * @return void
         */
        private function actions_for_refunded_order(): void
        {
            // Logging an unpaid order
            $this->logger->debug(sprintf(
                __FUNCTION__ . '. ' . __('Refund of payment.', 'wc-vtbpay') . '; Order ID: %s',
                $this->order->get_id()
            ));

            $this->order->update_status('refunded', __('Refund of payment.', 'wc-vtbpay'));
            wc_add_notice(__('The payment was refunded. The payment for the order was refunded through the personal account.', 'wc-vtbpay'), 'error');
        }


        /**
         * Получение статуса заказа WooCommerce из VTB API.
         *
         * @return string Значение статуса заказа, если оно существует, или пустая строка в противном случае.
         */
        private function get_payment_status(): string
        {
            $order_id = $this->order->get_id();

            // Logging input
            $this->logger->debug(
                __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
                'arguments' => [
                    'order_id' => $order_id
                ]
            ]);

            $response = $this->get_vtb_api()->getOrderInfo($order_id);

            // Logging output
    				$this->logger->debug(
                __FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
    						'response' => $response
    				]);

            return $response['object']['status']['value'] ?? '';
        }


        /**
         * Инициализация и настройка объекта класса VtbPayLogger.
         *
         * Эта функция инициализирует и настраивает логгер, используемый плагином VtbPay для ведения журнала.
         *
         * @return void
         */
    		private function set_vtbpay_logger(): void
        {
            if (version_compare(WOOCOMMERCE_VERSION, '2.1', '<')) {
                global $woocommerce;
                $wc_log = $woocommerce->logger();
            }
            else {
                $wc_log = new WC_Logger();
            }

            // Настройка в админке (логироать или нет)
            $logging = $this->logging_vtbpay;

            $this->logger = VtbPayLogger::getInstance()
                            ->setOption('showCurrentDate', false)
                            ->setOption('showLogLevel', false)
                            ->setCustomRecording(function($message) use ($wc_log){
                                $wc_log->error($message);
                            }, VtbPayLogger::LOG_LEVEL_ERROR)
                            ->setCustomRecording(function($message) use ($wc_log, $logging){
                                if ($logging) $wc_log->debug($message);
                            }, VtbPayLogger::LOG_LEVEL_DEBUG);
        }
    }
}
